/*
 * Copyright (C) 2015 ~ 2017 Deepin Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QVBoxLayout>
#include "simplelistviewtab.h"
#include "singlelistview.h"
#include "multilistview.h"

SimpleListViewTab::SimpleListViewTab(QWidget *parent) : QWidget(parent)
{
    QHBoxLayout *layout = new QHBoxLayout(this);
    //SingleListView *singleListView = new SingleListView();
    MultiListView *multiListView = new MultiListView();

    //layout->addWidget(singleListView);
    layout->addWidget(multiListView);
}

SimpleListViewTab::~SimpleListViewTab()
{
}
